#include "index.h"
#include "shape.h"

const char *shape_mxfields[2] = {"dd","ss"};

Shape shape_new(int nn) {
  Shape oo;
  oo.nn = nn;
  oo.dd = mxMalloc(sizeof(int) * oo.nn);
  oo.ss = mxMalloc(sizeof(int) * oo.nn);
  return oo;
}

void shape_free(Shape ii) {
  mxFree(ii.dd);
  mxFree(ii.ss);
}

Shape shape_copy(Shape ii) {
  Shape oo;
  int kk;
  oo = shape_new(ii.nn);
  for ( kk = 0 ; kk < oo.nn ; kk ++ ) {
    oo.dd[kk] = ii.dd[kk];
    oo.ss[kk] = ii.ss[kk];
  }
  return oo;
}

Shape shape_readindex(Index ii, int *ss) {
  Shape oo;
  int kk;
  oo = shape_new(ii.nn);
  for ( kk = 0 ; kk < oo.nn ; kk ++ ) {
    oo.dd[kk] = ii.dd[kk];
    oo.ss[kk] = ss[kk];
  }
  return oo;
}

Index shape_writeindex(Shape ii) {
  return index_copy(shape_index(ii));
}

Index shape_index(Shape ii) {
  Index oo;
  oo.nn = ii.nn;
  oo.dd = ii.dd;
  return oo;
}

Shape shape_readmxarray(const mxArray *marray) {
  Shape oo;
  double *dp, *sp;
  int kk, ll, nn, mm; 
  const int *ss;
  if ( mxIsClass(marray,"shape") || mxIsClass(marray,"table") ) {
    oo = shape_new(mxGetNumberOfElements(mxGetField(marray,0,"dd")));
    dp = mxGetPr(mxGetField(marray,0,"dd"));
    sp = mxGetPr(mxGetField(marray,0,"ss"));
    for ( kk = 0 ; kk < oo.nn ; kk ++ ) {
      oo.dd[kk] = * ( dp ++ );
      oo.ss[kk] = * ( sp ++ );
    }
  } else {
    mm = nn = mxGetNumberOfDimensions(marray);
    ss = mxGetDimensions(marray);
    for ( kk = 0 ; kk < nn ; kk ++ ) {
      if ( ss[kk] == 1 ) {
        mm --;
      }
    }
    oo = shape_new(mm);
    ll = 0;
    for ( kk = 0 ; kk < nn ; kk ++ ) {
      if ( ss[kk] != 1 ) {
        oo.dd[ll]  = kk;
        oo.ss[ll] = ss[kk];
        ll ++;
      }
    }
  }
  return oo;
}

mxArray *shape_writemxarray(Shape ii) {
  mxArray *marray, *oo;
  marray = mxCreateStructMatrix(1,1,2,shape_mxfields);
  shape_rewritemxarray(marray,ii);
  mexCallMATLAB(1,&oo,1,&marray,"shape");
  mxDestroyArray(marray);
  return oo;
}

void shape_rewritemxarray(mxArray *marray, Shape ii) {
  mxArray *de, *se;
  double *dp, *sp;
  int kk;

  if ( ( de = mxGetField(marray,0,"dd") ) != NULL ) mxDestroyArray(de);
  if ( ( de = mxGetField(marray,0,"ss") ) != NULL ) mxDestroyArray(de);
  de = mxCreateDoubleMatrix(1,ii.nn,mxREAL);
  se = mxCreateDoubleMatrix(1,ii.nn,mxREAL);
  dp = mxGetPr(de);
  sp = mxGetPr(se);
  for ( kk = 0 ; kk < ii.nn ; kk ++ ) {
    * ( dp ++ ) = ii.dd[kk];
    * ( sp ++ ) = ii.ss[kk];
  }
  mxSetField(marray,0,"dd",de);
  mxSetField(marray,0,"ss",se);
}

int shape_numelem(Shape ii) {
  int kk, oo;
  oo = 1;
  for ( kk = 0 ; kk < ii.nn ; kk ++ ) {
    oo *= ii.ss[kk];
  }
  return oo;
}

int shape_length(Shape ii) {
  return ( ii.nn );
}

bool shape_isempty(Shape ii) {
  return ( ii.nn == 0 );
}

bool shape_consistent(Shape ii, Shape jj) {
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp = jj.ss; jdend = jj.dd + jj.nn; 

  while (1) {
    if ( idp == idend || jdp == jdend ) break;
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      if ( *isp != 1 && *jsp != 1 && *isp != *jsp ) return 0; 
      idp ++; isp ++;
      jdp ++; jsp ++;
    } else if ( id < jd ) {
      idp ++; isp ++;
    } else {
      jdp ++; jsp ++;
    }
  }
  return 1;
}

bool shape_subshape(Shape ii, Shape jj) {
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp = jj.ss; jdend = jj.dd + jj.nn; 

  while (1) {
    if ( idp == idend ) {
      return 1;
    } else if ( jdp == jdend ) {
      while ( idp < idend ) {
        idp ++;
        if ( * ( isp ++ ) != 1 ) 
          return 0;
      }
      return 1;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      if ( ( * ( isp ) != 1 ) && ( * ( isp ++ ) != * ( jsp ++ ) ) ) 
        return 0;
      idp ++; jdp ++;
    } else if ( id < jd ) {
      if ( * ( isp ++ ) != 1 )
        return 0;
      idp ++;
    } else {
      jdp ++; jsp ++;
    }
  }
}

Shape shape_trim(Shape ii) {
  Shape oo;
  int nn, jj, kk;
  nn = 0;
  for ( kk = 0 ; kk < ii.nn ; kk ++ ) {
    nn += ( ii.ss[kk] != 1 );
  }
  oo = shape_new(nn);
  jj = 0;
  for ( kk = 0 ; kk < ii.nn ; kk ++ ) {
    if ( ii.ss[kk] != 1 ) {
      oo.dd[jj] = ii.dd[kk];
      oo.ss[jj] = ii.ss[kk];
      jj ++;
    }
  }
  return oo;
}

Shape shape_union(Shape ii, Shape jj) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp = jj.ss; jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      break;
    } else if ( jdp == jdend ) {
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp != 1 ? * isp : * jsp;
      idp ++; isp ++;
      jdp ++; jsp ++;
    } else if ( id < jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      idp ++; isp ++;
    } else {
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      jdp ++; jsp ++;
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_intersect(Shape ii, Shape jj) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp = jj.ss; jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      /*
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      */
      break;
    } else if ( jdp == jdend ) {
      /*
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      */
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp == 1 ? * isp : * jsp;
      idp ++; isp ++;
      jdp ++; jsp ++;
    } else if ( id < jd ) {
      /*
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      */
      idp ++; isp ++;
    } else {
      /*
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      */
      jdp ++; jsp ++;
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_diff(Shape ii, Shape jj) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp = jj.ss; jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      /*
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      */
      break;
    } else if ( jdp == jdend ) {
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      if ( * jsp == 1 ) {
        * ( odp ++ ) = id;
        * ( osp ++ ) = * isp;
      }
      idp ++; isp ++;
      jdp ++; jsp ++;
    } else if ( id < jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      idp ++; isp ++;
    } else {
      /*
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      */
      jdp ++; jsp ++;
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_xor(Shape ii, Shape jj) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp = jj.ss; jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      break;
    } else if ( jdp == jdend ) {
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      if ( * isp == 1 ) {
        * ( odp ++ ) = jd;
        * ( osp ++ ) = * jsp;
      } else if ( * jsp == 1 ) {
        * ( odp ++ ) = id;
        * ( osp ++ ) = * isp;
      }
      idp ++; isp ++;
      jdp ++; jsp ++;
    } else if ( id < jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      idp ++; isp ++;
    } else {
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      jdp ++; jsp ++;
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_indexdiff(Shape ii, Index jj) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp,       *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd;                jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      /*
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      */
      break;
    } else if ( jdp == jdend ) {
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      /*
      if ( * jsp == 1 ) {
        * ( odp ++ ) = id;
        * ( osp ++ ) = * isp;
      }
      */
      idp ++; isp ++;
      jdp ++; 
    } else if ( id < jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      idp ++; isp ++;
    } else {
      /*
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      */
      jdp ++; 
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_indexintersect(Shape ii, Index jj) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp,       *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd;                jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      /*
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      */
      break;
    } else if ( jdp == jdend ) {
      /*
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      */
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      idp ++; isp ++;
      jdp ++; 
    } else if ( id < jd ) {
      /*
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      */
      idp ++; isp ++;
    } else {
      /*
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      */
      jdp ++; 
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_asgn(Shape ii, Index jj, int * ss) {
  Shape oo;
  int *idp, *isp, *idend, id;
  int *jdp, *jsp, *jdend, jd;
  int *odp, *osp;

  oo = shape_new(ii.nn + jj.nn);

  idp = ii.dd; isp = ii.ss; idend = ii.dd + ii.nn;   
  jdp = jj.dd; jsp =    ss; jdend = jj.dd + jj.nn; 
  odp = oo.dd; osp = oo.ss;

  while (1) {
    if ( idp == idend ) {
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = * ( jsp ++ );
      }
      break;
    } else if ( jdp == jdend ) {
      while ( idp < idend ) {
        * ( odp ++ ) = * ( idp ++ );
        * ( osp ++ ) = * ( isp ++ );
      }
      break;
    }
    id = * idp;
    jd = * jdp;
    if ( id == jd ) {
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      idp ++; isp ++;
      jdp ++; jsp ++;
    } else if ( id < jd ) {
      * ( odp ++ ) = id;
      * ( osp ++ ) = * isp;
      idp ++; isp ++;
    } else {
      * ( odp ++ ) = jd;
      * ( osp ++ ) = * jsp;
      jdp ++; jsp ++;
    }
  }
  oo.nn  = (int) ( odp - oo.dd );
  oo.dd  = mxRealloc(oo.dd, sizeof(int)*oo.nn);
  oo.ss = mxRealloc(oo.ss, sizeof(int)*oo.nn);
  return oo;
}

Shape shape_ref(Shape ii, Index jj) {
  Shape oo;
  int *idp, *isp, *idend, *jdp, *jdend, *osp, *odp, id, jd;

  isp   = ii.ss;
  idp   = ii.dd;
  idend = idp + ii.nn;
  oo    = shape_new(jj.nn);
  osp   = oo.ss;
  odp   = oo.dd;
  jdp   = jj.dd;
  jdend = jdp + jj.nn;

  while (1) {
    if ( idp == idend ) {
      while ( jdp < jdend ) {
        * ( odp ++ ) = * ( jdp ++ );
        * ( osp ++ ) = 1;
      }
      return oo;
    } else if ( jdp == jdend ) {
      return oo;
    } else {
      id = * idp;
      jd = * jdp;
      if ( id == jd ) {
        * ( odp ++ ) = id;
        * ( osp ++ ) = * ( isp ++ );
        idp ++; jdp ++;
      } else if ( id < jd ) {
        idp ++; isp ++;
      } else if ( id > jd ) {
        * ( odp ++ ) = jd;
        * ( osp ++ ) = 1;
        jdp ++;
      }
    }
  }
}

